@@ -15,7 +15,7 @@ from recipemd._vendor.commonmark_extensions.plaintext import CommonMarkToCommonM
 from dataclasses_json import dataclass_json, config
 
 __all__ = ['RecipeParser', 'RecipeSerializer', 'multiply_recipe', 'get_recipe_with_yield',
-           'Recipe', 'Ingredient', 'IngredientGroup', 'Amount']
+           'Recipe', 'Ingredient', 'IngredientGroup', 'Amount', 'IngredientList']
 
 
 T = TypeVar('T')
@@ -129,6 +129,12 @@ class RecipeSerializer:
 
 
 class RecipeParser:
+    """
+    Parses strings to a :class:`Recipe` or :class:`Amount`.
+
+    The markdown format is described in the :ref:`RecipeMD Specification`. 
+    """
+
     _list_split = re.compile(r"(?<!\d),|,(?!\d)")
 
     src: Optional[str]
@@ -143,6 +149,31 @@ class RecipeParser:
         self.current = None
 
     def parse(self, src: str) -> Recipe:
+        """
+        Parses a markdown string into a :class:`Recipe`.
+
+        >>> recipe_parser = RecipeParser()
+        <BLANKLINE>
+        >>> recipe = recipe_parser.parse('''
+        ...   # Guacamole
+        ...   ---
+        ...   - *1* avocado
+        ...   - *.5 teaspoon* salt
+        ...   - *1 1/2 pinches* red pepper flakes
+        ...   - lemon juice
+        ...   ---
+        ...   Remove flesh from avocado and roughly mash with fork. Season to taste with salt pepper and lemon juice.
+        ... ''')
+        <BLANKLINE>
+        >>> recipe.title
+        'Guacamole'
+        <BLANKLINE>
+        >>> recipe.ingredients[0].name
+        'avocado'
+    
+
+        :raises RuntimeException: If src is not a valid RecipeMD recipe.
+        """
         self.src = src
         self.recipe = Recipe(title=None)
 
@@ -192,11 +223,15 @@ class RecipeParser:
     def _parse_tags_and_yields(self):
         while self.current is not None and (self._is_tags(self.current) or self._is_yields(self.current)):
             if self._is_tags(self.current):
+                if self.recipe.tags:
+                    raise RuntimeError(f"Invalid, tags may not be specified multiple times")
                 self._enter_node()
                 tags_text = self._get_current_node_children_source()
                 self._exit_node()
                 self.recipe = replace(self.recipe, tags=[t.strip() for t in self._list_split.split(tags_text)])
             else:
+                if self.recipe.yields:
+                    raise RuntimeError(f"Invalid, tags may not be specified multiple times")
                 self._enter_node()
                 yields_text = self._get_current_node_children_source()
                 self._exit_node()
@@ -317,6 +352,23 @@ class RecipeParser:
 
     @staticmethod
     def parse_amount(amount_str: str) -> Amount:
+        """
+        Parses an amount string to an :class:`Amount`.
+
+        >>> RecipeParser.parse_amount('3.5 l')
+        Amount(factor=Decimal('3.5'), unit='l')
+
+        Will recognize different :ref:`number formats<Amount>`:  
+
+        >>> RecipeParser.parse_amount('3 1/2 l')
+        Amount(factor=Decimal('3.5'), unit='l')
+        <BLANKLINE>
+        >>> RecipeParser.parse_amount('3 ½ l')
+        Amount(factor=Decimal('3.5'), unit='l')
+        <BLANKLINE>
+        >>> RecipeParser.parse_amount('3,5 l')
+        Amount(factor=Decimal('3.5'), unit='l')
+        """
         # iterate over different value format
         for regexp, factor_function, group_count in RecipeParser._value_formats:
             match = re.match(r'^\s*(-?)\s*' + regexp + r'(.*)$', amount_str)
@@ -360,13 +412,13 @@ class RecipeParser:
             self._next_node()
 
         if end is not None:
-            return self._get_sourcepos_source((start, end))
+            return self._get_sourcepos_source_lines((start, end))
 
         return None
 
     def _get_node_source(self, ast_node):
         if ast_node.sourcepos is not None:
-            return self._get_sourcepos_source(ast_node.sourcepos)
+            return self._get_sourcepos_source_lines(ast_node.sourcepos)
         if ast_node.literal is not None:
             return ast_node.literal
         return CommonMarkToCommonMarkRenderer().render(ast_node)
@@ -381,37 +433,52 @@ class RecipeParser:
             self._exit_node()
         return source
 
-    def _get_sourcepos_source(self, sourcepos):
+    def _get_sourcepos_source_lines(self, sourcepos):
         start_line = sourcepos[0][0] - 1
         end_line = sourcepos[1][0]
 
-        first_line_start_offset = sourcepos[0][1] - 1
-        last_line_end_offset = sourcepos[1][1]
-
         lines = self.src.splitlines()[start_line:end_line]
-        lines[0] = lines[0][first_line_start_offset:]
-        lines[-1] = lines[-1][:last_line_end_offset]
 
         return "\n".join(lines)
 
 
 def multiply_recipe(recipe: Recipe, multiplier: Decimal) -> Recipe:
     """
-    Multiplies a recipe by the given multiplier
+    Multiplies a recipe by the given multiplier.
 
-    Creates a new recipe where the factor of yield and ingredient is changed according to the multiplier
+    Creates a new recipe where the factor of yield and ingredient is changed according to the multiplier.
+
+    >>> recipe = Recipe(
+    ...   ingredients=[
+    ...     Ingredient(name='Eggs', amount=Amount(factor=Decimal('5'), unit=None), link=None),
+    ...     Ingredient(name='Butter', amount=Amount(factor=Decimal('200'), unit='g'), link=None),
+    ...    ]    
+    ... )
+    >>> multiplied_recipe = multiply_recipe(recipe, 3)
+    <BLANKLINE>
+    >>> multiplied_recipe.ingredients[0]
+    Ingredient(name='Eggs', amount=Amount(factor=Decimal('15'), unit=None), link=None)
+    <BLANKLINE>
+    >>> multiplied_recipe.ingredients[1]
+    Ingredient(name='Butter', amount=Amount(factor=Decimal('600'), unit='g'), link=None)
     """
-    recipe = replace(recipe, yields=[replace(y, factor=y.factor * multiplier) for y in recipe.yields])
+    recipe = replace(recipe, yields=[replace(y, factor=y.factor * multiplier) for y in recipe.yields if y.factor is not None])
     recipe = _multiply_ingredient_list(recipe, multiplier)
     return recipe
 
 
 def get_recipe_with_yield(recipe: Recipe, required_yield: Amount) -> Recipe:
     """
-    Scale the given recipe to a required yield
+    Scale the given recipe to a required yield.
 
-    Raises `StopIteration` if no yield with a matching unit can be found.
+    Creates a new recipe, which has the yield given by `required_yield`. A recipe can only be scaled if a yield with a matching 
+    unit is present.
+
+    :raises StopIteration: If no yield with a matching unit can be found.
+    :raises RuntimeError: If required_yield or the matching yield in the recipe do not have a factor.
     """
+    if required_yield.factor is None:
+        raise RuntimeError("Required yield must contain a factor")
     matching_recipe_yield = next((y for y in recipe.yields if y.unit == required_yield.unit), None)
     if matching_recipe_yield is None:
         # no unit in required amount is interpreted as "one recipe"
@@ -419,6 +486,8 @@ def get_recipe_with_yield(recipe: Recipe, required_yield: Amount) -> Recipe:
             matching_recipe_yield = Amount(Decimal(1))
         else:
             raise StopIteration
+    if matching_recipe_yield.factor is None:
+        raise RuntimeError(f"Recipe yield with matching unit must contain a factor")
     recipe = multiply_recipe(recipe, required_yield.factor / matching_recipe_yield.factor)
     return recipe
 
